@@ -348,8 +348,10 @@
 /*	time limit for sending a complete SMTP request and for receiving a
 /*	complete SMTP response.
 /* .IP "\fBsmtp_min_data_rate (500)\fR"
-/*	The minimum data transfer rate for DATA requests, when deadlines
-/*	are enabled with smtp_per_request_deadline.
+/*	The minimum plaintext data transfer rate in bytes/second for
+/*	DATA requests, when deadlines are enabled with smtp_per_request_deadline.
+/* .IP "\fBheader_from_format (standard)\fR"
+/*	The format of the Postfix-generated \fBFrom:\fR header.
 /* MIME PROCESSING CONTROLS
 /* .ad
 /* .fi
@@ -424,9 +426,7 @@
 /*	Detailed information about STARTTLS configuration may be found
 /*	in the TLS_README document.
 /* .IP "\fBsmtp_tls_security_level (empty)\fR"
-/*	The default SMTP TLS security level for the Postfix SMTP client;
-/*	when a non-empty value is specified, this overrides the obsolete
-/*	parameters smtp_use_tls, smtp_enforce_tls, and smtp_tls_enforce_peername.
+/*	The default SMTP TLS security level for the Postfix SMTP client.
 /* .IP "\fBsmtp_sasl_tls_security_options ($smtp_sasl_security_options)\fR"
 /*	The SASL authentication security options that the Postfix SMTP
 /*	client uses for TLS encrypted SMTP sessions.
@@ -559,7 +559,7 @@
 /*	Available in Postfix version 3.0 and later:
 /* .IP "\fBsmtp_tls_wrappermode (no)\fR"
 /*	Request that the Postfix SMTP client connects using the
-/*	legacy SMTPS protocol instead of using the STARTTLS command.
+/*	SUBMISSIONS/SMTPS protocol instead of using the STARTTLS command.
 /* .PP
 /*	Available in Postfix version 3.1 and later:
 /* .IP "\fBsmtp_tls_dane_insecure_mx_policy (see 'postconf -d' output)\fR"
@@ -698,8 +698,8 @@
 /*	time limit for sending a complete SMTP request and for receiving a
 /*	complete SMTP response.
 /* .IP "\fBsmtp_min_data_rate (500)\fR"
-/*	The minimum data transfer rate for DATA requests, when deadlines
-/*	are enabled with smtp_per_request_deadline.
+/*	The minimum plaintext data transfer rate in bytes/second for
+/*	DATA requests, when deadlines are enabled with smtp_per_request_deadline.
 /* .PP
 /*	Implemented in the qmgr(8) daemon:
 /* .IP "\fBtransport_destination_concurrency_limit ($default_destination_concurrency_limit)\fR"
@@ -718,7 +718,7 @@
 /*	Preliminary SMTPUTF8 support is introduced with Postfix 3.0.
 /* .IP "\fBsmtputf8_enable (yes)\fR"
 /*	Enable preliminary SMTPUTF8 support for the protocols described
-/*	in RFC 6531..6533.
+/*	in RFC 6531, RFC 6532, and RFC 6533.
 /* .IP "\fBsmtputf8_autodetect_classes (sendmail, verify)\fR"
 /*	Detect that a message requires SMTPUTF8 support for the specified
 /*	mail origin classes.
@@ -847,6 +847,11 @@
 /*	Available in Postfix 3.3 and later:
 /* .IP "\fBservice_name (read-only)\fR"
 /*	The master.cf service name of a Postfix daemon process.
+/* .PP
+/*	Available in Postfix 3.7 and later:
+/* .IP "\fBsmtp_bind_address_enforce (no)\fR"
+/*	Defer delivery when the Postfix SMTP client cannot apply the
+/*	smtp_bind_address or smtp_bind_address6 setting.
 /* SEE ALSO
 /*	generic(5), output address rewriting
 /*	header_checks(5), message header content inspection
@@ -941,6 +946,7 @@
 #include <string_list.h>
 #include <maps.h>
 #include <ext_prop.h>
+#include <hfrom_format.h>
 
 /* DNS library. */
 
@@ -1083,6 +1089,9 @@ char   *var_smtp_sasl_auth_cache_name;
 int     var_smtp_sasl_auth_cache_time;
 bool    var_smtp_sasl_auth_soft_bounce;
 
+char   *var_hfrom_format;
+bool var_smtp_bind_addr_enforce;
+
  /*
   * Global variables.
   */
@@ -1099,6 +1108,7 @@ MAPS   *smtp_pix_bug_maps;
 HBC_CHECKS *smtp_header_checks;		/* limited header checks */
 HBC_CHECKS *smtp_body_checks;		/* limited body checks */
 SMTP_CLI_ATTR smtp_cli_attr;		/* parsed command-line */
+int     smtp_hfrom_format;		/* postmaster notifications */
 
 #ifdef USE_TLS
 
@@ -1384,6 +1394,11 @@ static void post_init(char *unused_name, char **argv)
      * the process lifetime.
      */
     get_cli_attr(&smtp_cli_attr, argv);
+
+    /*
+     * header_from format, for postmaster notifications.
+     */
+    smtp_hfrom_format = hfrom_format_parse(VAR_HFROM_FORMAT, var_hfrom_format);
 }
 
 /* pre_init - pre-jail initialization */
@@ -1618,7 +1633,7 @@ int     main(int argc, char **argv)
 		       CA_MAIL_SERVER_BOOL_TABLE(smtp_mode ?
 					 smtp_bool_table : lmtp_bool_table),
 		       CA_MAIL_SERVER_NBOOL_TABLE(smtp_mode ?
-					 smtp_nbool_table : lmtp_nbool_table),
+				       smtp_nbool_table : lmtp_nbool_table),
 		       CA_MAIL_SERVER_PRE_INIT(pre_init),
 		       CA_MAIL_SERVER_POST_INIT(post_init),
 		       CA_MAIL_SERVER_PRE_ACCEPT(pre_accept),
