@@ -39,13 +39,23 @@
 /*	void	dict_close(dict)
 /*	DICT	*dict;
 /*
+/*	typedef struct {
+/* .in +4
+/*	    char   *type;
+/*	    DICT_OPEN_FN dict_fn;
+/*	    MKMAP_OPEN_FN mkmap_fn;	/* See <mkmap.h> */
+/* .in -4
+/*	} DICT_OPEN_INFO;
+/*
 /*	typedef DICT *(*DICT_OPEN_FN) (const char *, int, int);
 /*
-/*	dict_open_register(type, open)
-/*	const char *type;
-/*	DICT_OPEN_FN open;
+/*	void	dict_open_register(open_info)
+/*	DICT_OPEN_INFO *open_info;
 /*
-/*	typedef DICT_OPEN_FN (*DICT_OPEN_EXTEND_FN)(const char *type);
+/*	const DICT_OPEN_INFO *dict_open_lookup(dict_type)
+/*	const char *dict_type;
+/*
+/*	typedef DICT_OPEN_INFO (*DICT_OPEN_EXTEND_FN)(char *);
 /*
 /*	DICT_OPEN_EXTEND_FN dict_open_extend(call_back)
 /*	DICT_OPEN_EXTEND_FN call_back;
@@ -238,6 +248,11 @@
 /*	associated data structures.
 /*
 /*	dict_open_register() adds support for a new dictionary type.
+/*	NOTE: this function does not copy its argument.
+/*
+/*	dict_open_lookup() returns a pointer to the DICT_OPEN_INFO
+/*	for the specified dictionary type, or a null pointer if the
+/*	requested information is not found.
 /*
 /*	dict_open_extend() registers a call-back function that looks
 /*	up the dictionary open() function for a type that is not
@@ -348,60 +363,56 @@
 #include <split_at.h>
 #include <htable.h>
 #include <myflock.h>
+#include <mkmap.h>
 
  /*
   * lookup table for available map types.
   */
-typedef struct {
-    char   *type;
-    DICT_OPEN_FN open;
-} DICT_OPEN_INFO;
-
 static const DICT_OPEN_INFO dict_open_info[] = {
-    DICT_TYPE_ENVIRON, dict_env_open,
-    DICT_TYPE_HT, dict_ht_open,
-    DICT_TYPE_UNIX, dict_unix_open,
-    DICT_TYPE_TCP, dict_tcp_open,
+    DICT_TYPE_ENVIRON, dict_env_open, 0,
+    DICT_TYPE_HT, dict_ht_open, 0,
+    DICT_TYPE_UNIX, dict_unix_open, 0,
+    DICT_TYPE_TCP, dict_tcp_open, 0,
 #ifdef HAS_DBM
-    DICT_TYPE_DBM, dict_dbm_open,
+    DICT_TYPE_DBM, dict_dbm_open, mkmap_dbm_open,
 #endif
 #ifdef HAS_DB
-    DICT_TYPE_HASH, dict_hash_open,
-    DICT_TYPE_BTREE, dict_btree_open,
+    DICT_TYPE_HASH, dict_hash_open, mkmap_hash_open,
+    DICT_TYPE_BTREE, dict_btree_open, mkmap_btree_open,
 #endif
 #ifdef HAS_NIS
-    DICT_TYPE_NIS, dict_nis_open,
+    DICT_TYPE_NIS, dict_nis_open, 0,
 #endif
 #ifdef HAS_NISPLUS
-    DICT_TYPE_NISPLUS, dict_nisplus_open,
+    DICT_TYPE_NISPLUS, dict_nisplus_open, 0,
 #endif
 #ifdef HAS_NETINFO
-    DICT_TYPE_NETINFO, dict_ni_open,
+    DICT_TYPE_NETINFO, dict_ni_open, 0,
 #endif
 #ifdef HAS_POSIX_REGEXP
-    DICT_TYPE_REGEXP, dict_regexp_open,
+    DICT_TYPE_REGEXP, dict_regexp_open, 0,
 #endif
-    DICT_TYPE_STATIC, dict_static_open,
-    DICT_TYPE_CIDR, dict_cidr_open,
-    DICT_TYPE_THASH, dict_thash_open,
-    DICT_TYPE_SOCKMAP, dict_sockmap_open,
-    DICT_TYPE_FAIL, dict_fail_open,
-    DICT_TYPE_PIPE, dict_pipe_open,
-    DICT_TYPE_RANDOM, dict_random_open,
-    DICT_TYPE_UNION, dict_union_open,
-    DICT_TYPE_INLINE, dict_inline_open,
+    DICT_TYPE_STATIC, dict_static_open, 0,
+    DICT_TYPE_CIDR, dict_cidr_open, 0,
+    DICT_TYPE_THASH, dict_thash_open, 0,
+    DICT_TYPE_SOCKMAP, dict_sockmap_open, 0,
+    DICT_TYPE_FAIL, dict_fail_open, mkmap_fail_open,
+    DICT_TYPE_PIPE, dict_pipe_open, 0,
+    DICT_TYPE_RANDOM, dict_random_open, 0,
+    DICT_TYPE_UNION, dict_union_open, 0,
+    DICT_TYPE_INLINE, dict_inline_open, 0,
 #ifndef USE_DYNAMIC_MAPS
-#if defined(HAS_PCRE) || defined(HAS_PCRE2)
-    DICT_TYPE_PCRE, dict_pcre_open,
+#ifdef HAS_PCRE
+    DICT_TYPE_PCRE, dict_pcre_open, 0,
 #endif
 #ifdef HAS_CDB
-    DICT_TYPE_CDB, dict_cdb_open,
+    DICT_TYPE_CDB, dict_cdb_open, mkmap_cdb_open,
 #endif
 #ifdef HAS_SDBM
-    DICT_TYPE_SDBM, dict_sdbm_open,
+    DICT_TYPE_SDBM, dict_sdbm_open, mkmap_sdbm_open,
 #endif
 #ifdef HAS_LMDB
-    DICT_TYPE_LMDB, dict_lmdb_open,
+    DICT_TYPE_LMDB, dict_lmdb_open, mkmap_lmdb_open,
 #endif
 #endif					/* !USE_DYNAMIC_MAPS */
     0,
@@ -453,15 +464,13 @@ DICT   *dict_open(const char *dict_spec, int open_flags, int dict_flags)
     return (dict);
 }
 
-
 /* dict_open3 - open dictionary */
 
 DICT   *dict_open3(const char *dict_type, const char *dict_name,
 		           int open_flags, int dict_flags)
 {
     const char *myname = "dict_open";
-    DICT_OPEN_INFO *dp;
-    DICT_OPEN_FN open_fn;
+    const DICT_OPEN_INFO *dp;
     DICT   *dict;
 
     if (*dict_type == 0 || *dict_name == 0)
@@ -469,17 +478,10 @@ DICT   *dict_open3(const char *dict_type, const char *dict_name,
 		  dict_type, dict_name);
     if (dict_open_hash == 0)
 	dict_open_init();
-    if ((dp = (DICT_OPEN_INFO *) htable_find(dict_open_hash, dict_type)) == 0) {
-	if (dict_open_extend_hook != 0
-	    && (open_fn = dict_open_extend_hook(dict_type)) != 0) {
-	    dict_open_register(dict_type, open_fn);
-	    dp = (DICT_OPEN_INFO *) htable_find(dict_open_hash, dict_type);
-	}
-	if (dp == 0)
-	    return (dict_surrogate(dict_type, dict_name, open_flags, dict_flags,
+    if ((dp = dict_open_lookup(dict_type)) == 0)
+	return (dict_surrogate(dict_type, dict_name, open_flags, dict_flags,
 			     "unsupported dictionary type: %s", dict_type));
-    }
-    if ((dict = dp->open(dict_name, open_flags, dict_flags)) == 0)
+    if ((dict = dp->dict_fn(dict_name, open_flags, dict_flags)) == 0)
 	return (dict_surrogate(dict_type, dict_name, open_flags, dict_flags,
 			    "cannot open %s:%s: %m", dict_type, dict_name));
     if (msg_verbose)
@@ -508,20 +510,33 @@ DICT   *dict_open3(const char *dict_type, const char *dict_name,
 
 /* dict_open_register - register dictionary type */
 
-void    dict_open_register(const char *type, DICT_OPEN_FN open)
+void    dict_open_register(const DICT_OPEN_INFO *dp)
 {
     const char *myname = "dict_open_register";
-    DICT_OPEN_INFO *dp;
-    HTABLE_INFO *ht;
 
+    if (msg_verbose > 1)
+	msg_info("%s: %s", myname, dp->type);
     if (dict_open_hash == 0)
 	dict_open_init();
-    if (htable_find(dict_open_hash, type))
-	msg_panic("%s: dictionary type exists: %s", myname, type);
-    dp = (DICT_OPEN_INFO *) mymalloc(sizeof(*dp));
-    dp->open = open;
-    ht = htable_enter(dict_open_hash, type, (void *) dp);
-    dp->type = ht->key;
+    if (htable_find(dict_open_hash, dp->type))
+	msg_panic("%s: dictionary type exists: %s", myname, dp->type);
+    (void) htable_enter(dict_open_hash, dp->type, (void *) dp);
+}
+
+/* dict_open_lookup - look up DICT_OPEN_INFO for dictionary type */
+
+const DICT_OPEN_INFO *dict_open_lookup(const char *dict_type)
+{
+    const char myname[] = "dict_open_lookup";
+    const DICT_OPEN_INFO *dp;
+
+    if (msg_verbose > 1)
+	msg_info("%s: %s", myname, dict_type);
+    if ((dp = (DICT_OPEN_INFO *) htable_find(dict_open_hash, dict_type)) == 0
+	&& dict_open_extend_hook != 0
+	&& (dp = dict_open_extend_hook(dict_type)) != 0)
+	dict_open_register(dp);
+    return (dp);
 }
 
 /* dict_open_extend - register alternate dictionary search routine */
